@@ -2,6 +2,7 @@ import base64
 from typing import Any, Dict, List, Optional, Union
 
 from pydantic import error_wrappers
+from requests.exceptions import RequestException
 
 from marqo.index import Index
 from marqo.config import Config
@@ -9,6 +10,15 @@ from marqo.models import BulkSearchBody, BulkSearchQuery
 from marqo._httprequests import HttpRequests
 from marqo import utils, enums
 from marqo import errors
+from marqo.version import minimum_supported_marqo_version
+from marqo.marqo_logging import mq_logger
+from marqo.errors import MarqoWebError
+# we want to avoid name conflicts with marqo.version
+from packaging import version as versioning_helpers
+from json import JSONDecodeError
+
+# A dictionary to cache the marqo url and version for compatibility check
+marqo_url_and_version_cache = {}
 
 
 class Client:
@@ -18,11 +28,12 @@ class Client:
     A client instance is needed for every marqo API method to know the location of
     marqo and its permissions.
     """
+
     def __init__(
-        self, url: str = "http://localhost:8882", main_user: str = None, main_password: str = None,
-        indexing_device: Optional[Union[enums.Devices, str]] = None,
-        search_device: Optional[Union[enums.Devices, str]] = None,
-        api_key: str = None
+            self, url: str = "http://localhost:8882",
+            main_user: str = None, main_password: str = None,
+            return_telemetry: bool = False,
+            api_key: str = None
     ) -> None:
         """
         Parameters
@@ -36,8 +47,9 @@ class Client:
             self.url = utils.construct_authorized_url(url_base=url, username=main_user, password=main_password)
         else:
             self.url = url
-        self.config = Config(self.url, indexing_device=indexing_device, search_device=search_device, api_key=api_key)
+        self.config = Config(self.url, use_telemetry=return_telemetry, api_key=api_key)
         self.http = HttpRequests(self.config)
+        self._marqo_minimum_supported_version_check()
 
     def create_index(
         self, index_name: str,
@@ -46,9 +58,14 @@ class Client:
         sentences_per_chunk=2,
         sentence_overlap=0,
         image_preprocessing_method=None,
-        settings_dict=None
+        settings_dict=None,
+        inference_node_type=None,
+        storage_node_type=None,
+        inference_node_count=1,
+        storage_node_count=1,
+        replicas_count=0,
     ) -> Dict[str, Any]:
-        """Create the index.
+        """Create the index. Please refer to the marqo cloud to see options for inference and storage node types.
 
         Args:
             index_name: name of the index.
@@ -61,6 +78,11 @@ class Client:
             settings_dict: if specified, overwrites all other setting
                 parameters, and is passed directly as the index's
                 index_settings
+            inference_node_type:
+            storage_node_type:
+            inference_node_count;
+            storage_node_count:
+            replicas_count:
         Returns:
             Response body, containing information about index creation result
         """
@@ -70,7 +92,9 @@ class Client:
             model=model, normalize_embeddings=normalize_embeddings,
             sentences_per_chunk=sentences_per_chunk, sentence_overlap=sentence_overlap,
             image_preprocessing_method=image_preprocessing_method,
-            settings_dict=settings_dict
+            settings_dict=settings_dict, inference_node_type=inference_node_type, storage_node_type=storage_node_type,
+            storage_node_count=storage_node_count, replicas_count=replicas_count,
+            inference_node_count=inference_node_count,
         )
 
     def delete_index(self, index_name: str) -> Dict[str, Any]:
@@ -101,7 +125,7 @@ class Client:
         """
         ix = Index(self.config, index_name)
         # verify it exists:
-        self.http.get(path=f"indexes/{index_name}/stats")
+        self.http.get(path=f"indexes/{index_name}/stats", index_name=index_name)
         return ix
 
     def index(self, index_name: str) -> Index:
@@ -139,8 +163,7 @@ class Client:
 
     def enrich(self, documents: List[Dict], enrichment: Dict, device: str = None, ):
         """Enrich documents"""
-        selected_device = device if device is not None else self.config.indexing_device
-        translated = utils.translate_device_string_for_url(selected_device)
+        translated = utils.translate_device_string_for_url(device)
         response = self.http.post(path=f'enrichment?device={translated}', body={
             "documents": documents,
             "enrichment": enrichment
@@ -153,17 +176,15 @@ class Client:
         except error_wrappers.ValidationError as e:
             raise errors.InvalidArgError(f"some parameters in search query(s) are invalid. Errors are: {e.errors()}")
 
-        selected_device = device if device is not None else self.config.search_device
-        translated = utils.translate_device_string_for_url(selected_device)
-        
+        translated_device_param = f"{f'?&device={utils.translate_device_string_for_url(device)}' if device is not None else ''}"
         return self.http.post(
-            f"indexes/bulk/search?device={translated}",
+            f"indexes/bulk/search{translated_device_param}",
             body=BulkSearchQuery(queries=parsed_queries).json()
         )
 
     @staticmethod
     def _base64url_encode(
-        data: bytes
+            data: bytes
     ) -> str:
         return base64.urlsafe_b64encode(data).decode('utf-8').replace('=', '')
 
@@ -173,20 +194,50 @@ class Client:
     def health(self):
         return self.http.get(path="health")
 
-
-    def eject_model(self, model_name:str, model_device:str):
+    def eject_model(self, model_name: str, model_device: str):
         return self.http.delete(path=f"models?model_name={model_name}&model_device={model_device}")
 
-
     def get_loaded_models(self):
         return self.http.get(path="models")
 
-
     def get_cuda_info(self):
         return self.http.get(path="device/cuda")
 
-
     def get_cpu_info(self):
         return self.http.get(path="device/cpu")
 
+    def _marqo_minimum_supported_version_check(self):
+        min_ver = minimum_supported_marqo_version()
+        skip_warning_message = (
+            f"Marqo encountered a problem trying to check the Marqo version found at `{self.url}`. "
+            f"The minimum supported Marqo version for this client is {min_ver}. "
+            f"If you are sure your Marqo version is compatible with this client, you can ignore this message. ")
+
+        # Skip the check if the url is previously labelled as "_skipped"
+        if self.url in marqo_url_and_version_cache and marqo_url_and_version_cache[self.url] == "_skipped":
+            mq_logger.warning(skip_warning_message)
+            return
+
+        # Skip the check for Marqo CloudV2 APIs right now
+        skip_version_check_url = ["https://api.marqo.ai", "https://cloud.marqo.ai"]
+        if self.url in skip_version_check_url:
+            marqo_url_and_version_cache[self.url] = "_skipped"
+            mq_logger.warning(skip_warning_message)
+            return
+
+        # Do version check
+        try:
+            if self.url not in marqo_url_and_version_cache:
+                marqo_url_and_version_cache[self.url] = self.get_marqo()["version"]
+            marqo_version = marqo_url_and_version_cache[self.url]
+            if versioning_helpers.parse(marqo_version) < versioning_helpers.parse(min_ver):
+                mq_logger.warning(f"Your Marqo Python client requires a minimum Marqo version of "
+                                  f"{minimum_supported_marqo_version()} to function properly, but your Marqo version is {marqo_version}. "
+                                  f"Please upgrade your Marqo instance to avoid potential errors. "
+                                  f"If you have already changed your Marqo instance but still get this warning, please restart your Marqo client Python interpreter.")
+        except (MarqoWebError, RequestException, TypeError, KeyError) as e:
+            mq_logger.warning(skip_warning_message)
+            marqo_url_and_version_cache[self.url] = "_skipped"
+        return
+
 
