@@ -2,6 +2,8 @@ import functools
 import json
 import logging
 import pprint
+import time
+
 from marqo import defaults
 import typing
 from urllib import parse
@@ -54,7 +56,12 @@ class Index:
                sentences_per_chunk=2,
                sentence_overlap=0,
                image_preprocessing_method=None,
-               settings_dict: dict = None
+               settings_dict: dict = None,
+               inference_node_type: str = None,
+               storage_node_type: str = None,
+               inference_node_count: int = 1,
+               storage_node_count: int = 1,
+               replicas_count: int = 0,
                ) -> Dict[str, Any]:
         """Create the index.
 
@@ -70,6 +77,11 @@ class Index:
             settings_dict: if specified, overwrites all other setting
                 parameters, and is passed directly as the index's
                 index_settings
+            inference_node_type: inference type for the index
+            storage_node_type: storage type for the index
+            inference_node_count: number of inference nodes for the index
+            storage_node_count: number of storage nodes for the index
+            replicas_count: number of replicas for the index
         Returns:
             Response body, containing information about index creation result
         """
@@ -91,7 +103,21 @@ class Index:
             cl_text_preprocessing['split_length'] = sentences_per_chunk
             cl_img_preprocessing = cl_ix_defaults['image_preprocessing']
             cl_img_preprocessing['patch_method'] = image_preprocessing_method
-            return req.post(f"indexes/{index_name}", body=cl_settings)
+            if not config.cluster_is_marqo:
+                return req.post(f"indexes/{index_name}", body=cl_settings)
+            cl_settings['inference_type'] = inference_node_type
+            cl_settings['storage_class'] = storage_node_type
+            cl_settings['number_of_inferences'] = inference_node_count
+            cl_settings['number_of_replicas'] = replicas_count
+            cl_settings['number_of_shards'] = storage_node_count
+            response = req.post(f"indexes/{index_name}", body=cl_settings)
+            index = Index(config, index_name)
+            creation = index.get_status()
+            while creation['index_status'] != 'READY':
+                time.sleep(10)
+                creation = index.get_status()
+                mq_logger.info(f"Index creation status: {creation['index_status']}")
+            return response
 
         return req.post(f"indexes/{index_name}", body={
             "index_defaults": {
@@ -111,7 +137,11 @@ class Index:
 
     def refresh(self):
         """refreshes the index"""
-        return self.http.post(path=F"indexes/{self.index_name}/refresh")
+        return self.http.post(path=F"indexes/{self.index_name}/refresh", index_name=self.index_name,)
+
+    def get_status(self):
+        """gets the status of the index"""
+        return self.http.get(path=F"indexes/{self.index_name}/status")
 
     def search(self, q: Union[str, dict], searchable_attributes: Optional[List[str]] = None,
                limit: int = 10, offset: int = 0, search_method: Union[SearchMethods.TENSOR, str] = SearchMethods.TENSOR,
@@ -137,7 +167,7 @@ class Index:
             show_highlights: True if highlights are to be returned
             reranker:
             device: the device used to index the data. Examples include "cpu",
-                "cuda" and "cuda:2". Overrides the Client's default device.
+                "cuda" and "cuda:2".
             filter_string: a filter string, used to prefilter documents during the
                 search. For example: "car_colour:blue"
             attributes_to_retrieve: a list of document attributes to be
@@ -152,14 +182,13 @@ class Index:
 
         start_time_client_request = timer()
         if highlights is not None:
-            logging.warning("Deprecation warning for parameter 'highlights'. "
-                            "Please use the 'showHighlights' instead. ")
+            mq_logger.warning("Deprecation warning for parameter 'highlights'. "
+                              "Please use the 'showHighlights' instead. ")
             show_highlights = highlights if show_highlights is True else show_highlights
 
-        selected_device = device if device is not None else self.config.search_device
         path_with_query_str = (
-            f"indexes/{self.index_name}/search?"
-            f"&device={utils.translate_device_string_for_url(selected_device)}"
+            f"indexes/{self.index_name}/search"
+            f"{f'?&device={utils.translate_device_string_for_url(device)}' if device is not None else ''}"
         )
         body = {
             "q": q,
@@ -185,7 +214,8 @@ class Index:
             body["modelAuth"] = model_auth
         res = self.http.post(
             path=path_with_query_str,
-            body=body
+            body=body,
+            index_name=self.index_name,
         )
 
         num_results = len(res["hits"])
@@ -215,7 +245,7 @@ class Index:
         url_string = f"indexes/{self.index_name}/documents/{document_id}"
         if expose_facets is not None:
             url_string += f"?expose_facets={expose_facets}"
-        return self.http.get(url_string)
+        return self.http.get(url_string, index_name=self.index_name,)
 
     def get_documents(self, document_ids: List[str], expose_facets=None) -> Dict[str, Any]:
         """Gets a selection of documents based on their IDs.
@@ -234,17 +264,17 @@ class Index:
             url_string += f"?expose_facets={expose_facets}"
         return self.http.get(
             url_string,
-            body=document_ids
+            body=document_ids,
+            index_name=self.index_name,
         )
 
     def add_documents(
         self,
         documents: List[Dict[str, Any]],
-        auto_refresh=True,
-        server_batch_size: int = None,
+        auto_refresh: bool = True,
         client_batch_size: int = None,
-        processes: int = None,
         device: str = None,
+        tensor_fields: List[str] = None,
         non_tensor_fields: List[str] = None,
         use_existing_tensors: bool = False,
         image_download_headers: dict = None,
@@ -259,16 +289,16 @@ class Index:
             auto_refresh: Automatically refresh the index. If you are making
                 lots of requests, it is advised to set this to False to
                 increase performance.
-            server_batch_size: if it is set, documents will be indexed into batches
-                on the server as they are indexed. Otherwise documents are unbatched
-                server-side.
             client_batch_size: if it is set, documents will be indexed into batches
                 in the client, before being sent off. Otherwise documents are unbatched
                 client-side.
-            processes: number of processes for the server to use, to do indexing,
             device: the device used to index the data. Examples include "cpu",
                 "cuda" and "cuda:2"
-            non_tensor_fields: fields within documents to not create and store tensors against.
+            tensor_fields: fields within documents to create and store tensors against.
+            non_tensor_fields: fields within documents to not create and store tensors against. Cannot be used with
+                tensor_fields.
+                .. deprecated:: 2.0.0
+                    This parameter has been deprecated and will be removed in Marqo 2.0.0. User tensor_fields instead.
             use_existing_tensors: use vectors that already exist in the docs.
             image_download_headers: a dictionary of headers to be passed while downloading images,
                 for URLs found in documents
@@ -277,73 +307,35 @@ class Index:
         Returns:
             Response body outlining indexing result
         """
-        if non_tensor_fields is None:
-            non_tensor_fields = []
+        if tensor_fields is not None and non_tensor_fields is not None:
+            raise errors.InvalidArgError('Cannot define `non_tensor_fields` when `tensor_fields` is defined. '
+                                         '`non_tensor_fields` has been deprecated and will be removed in Marqo 2.0.0. '
+                                         'Its use is discouraged.')
+
+        if tensor_fields is None and non_tensor_fields is None:
+            raise errors.InvalidArgError('You must include the `tensor_fields` parameter. '
+                                         'Use `tensor_fields=[]` to index for lexical-only search.')
+
+        if non_tensor_fields is not None:
+            mq_logger.warning('The `non_tensor_fields` parameter has been deprecated and will be removed in '
+                              'Marqo 2.0.0. Use `tensor_fields` instead.')
+
         if image_download_headers is None:
             image_download_headers = dict()
-        return self._generic_add_update_docs(
-            update_method="replace",
-            documents=documents, auto_refresh=auto_refresh, server_batch_size=server_batch_size,
-            client_batch_size=client_batch_size, processes=processes, device=device, non_tensor_fields=non_tensor_fields,
-            use_existing_tensors=use_existing_tensors, image_download_headers=image_download_headers, mappings=mappings,
-            model_auth=model_auth
+        return self._add_docs_organiser(
+            documents=documents, auto_refresh=auto_refresh,
+            client_batch_size=client_batch_size, device=device, tensor_fields=tensor_fields,
+            non_tensor_fields=non_tensor_fields, use_existing_tensors=use_existing_tensors,
+            image_download_headers=image_download_headers, mappings=mappings, model_auth=model_auth
         )
 
-    def update_documents(
+    def _add_docs_organiser(
         self,
         documents: List[Dict[str, Any]],
         auto_refresh=True,
-        server_batch_size: int = None,
         client_batch_size: int = None,
-        processes: int = None,
-        device: str = None,
-        non_tensor_fields: List[str] = None,
-    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
-        """
-        Will be deprecated soon.
-        Add documents to this index. Does a partial updates on existing documents,
-        based on their ID. Adds unseen documents to the index.
-
-        Args:
-            documents: List of documents. Each document should be a dictionary.
-            auto_refresh: Automatically refresh the index. If you are making
-                lots of requests, it is advised to turn this to false to
-                increase performance.
-            server_batch_size: if it is set, documents will be indexed into batches
-                on the server as they are indexed. Otherwise documents are unbatched
-                server-side.
-            client_batch_size: if it is set, documents will be indexed into batches
-                in the client, before being sent of. Otherwise documents are unbatched
-                client-side.
-            processes: number of processes for the server to use, to do indexing,
-            device: the device used to index the data. Examples include "cpu",
-                "cuda" and "cuda:2"
-            non_tensor_fields: fields within documents to not create and store tensors against.
-
-        Returns:
-            Response body outlining indexing result
-        """
-        mq_logger.warning(
-            "`update_documents()` is deprecated and will be removed in a future release. "
-            "To update documents with minimal inference operations, use `add_documents()` "
-            "with use_existing_tensors=True ")
-        if non_tensor_fields is None:
-            non_tensor_fields = []
-        return self._generic_add_update_docs(
-            update_method="update",
-            documents=documents, auto_refresh=auto_refresh, server_batch_size=server_batch_size,
-            client_batch_size=client_batch_size, processes=processes, device=device, non_tensor_fields=non_tensor_fields,
-        )
-
-    def _generic_add_update_docs(
-        self,
-        update_method: str,
-        documents: List[Dict[str, Any]],
-        auto_refresh=True,
-        server_batch_size: int = None,
-        client_batch_size: int = None,
-        processes: int = None,
         device: str = None,
+        tensor_fields: List = None,
         non_tensor_fields: List = None,
         use_existing_tensors: bool = False,
         image_download_headers: dict = None,
@@ -351,33 +343,34 @@ class Index:
         model_auth: dict = None
     ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
 
+        if (tensor_fields is None and non_tensor_fields is None) \
+                or (tensor_fields is not None and non_tensor_fields is not None):
+            raise ValueError("Exactly one of tensor_fields or non_tensor_fields must be provided.")
+
         error_detected_message = ('Errors detected in add documents call. '
                                   'Please examine the returned result object for more information.')
-        if non_tensor_fields is None:
-            non_tensor_fields = []
 
-        selected_device = device if device is not None else self.config.indexing_device
         num_docs = len(documents)
 
         # ADD DOCS TIMER-LOGGER (1)
         t0 = timer()
         start_time_client_process = timer()
         base_path = f"indexes/{self.index_name}/documents"
-        non_tensor_fields_query_param = utils.convert_list_to_query_params("non_tensor_fields", non_tensor_fields)
-        image_download_headers_param = (utils.convert_dict_to_url_params(image_download_headers)
-                                        if image_download_headers else '')
-        model_auth_param = (utils.convert_dict_to_url_params(model_auth) if model_auth else '')
-        mappings_param = (utils.convert_dict_to_url_params(mappings) if mappings else '')
         query_str_params = (
-            f"{f'&device={utils.translate_device_string_for_url(selected_device)}'}"
-            f"{f'&processes={processes}' if processes is not None else ''}"
-            f"{f'&batch_size={server_batch_size}' if server_batch_size is not None else ''}"
-            f"{f'&use_existing_tensors={str(use_existing_tensors).lower()}' if use_existing_tensors is not None else ''}"
-            f"{f'&{non_tensor_fields_query_param}' if len(non_tensor_fields) > 0 else ''}"
-            f"{f'&image_download_headers={image_download_headers_param}' if image_download_headers else ''}"
-            f"{f'&mappings={mappings_param}' if mappings else ''}"
-            f"{f'&model_auth={model_auth_param}' if model_auth_param else ''}"
+            f"{f'&device={utils.translate_device_string_for_url(device)}' if device is not None else ''}"
         )
+
+        base_body = {
+            "useExistingTensors" : use_existing_tensors,
+            "imageDownloadHeaders" : image_download_headers,
+            "mappings" : mappings,
+            "modelAuth": model_auth,
+        }
+        if tensor_fields is not None:
+            base_body['tensorFields'] = tensor_fields
+        else:
+            base_body['nonTensorFields'] = non_tensor_fields
+
         end_time_client_process = timer()
         total_client_process_time = end_time_client_process - start_time_client_process
         mq_logger.debug(f"add_documents pre-processing: took {(total_client_process_time):.3f}s for {num_docs} docs, "
@@ -388,8 +381,8 @@ class Index:
                 raise errors.InvalidArgError("Batch size can't be less than 1!")
             res = self._batch_request(
                 base_path=base_path, auto_refresh=auto_refresh,
-                update_method=update_method, docs=documents, verbose=False,
-                query_str_params=query_str_params, batch_size=client_batch_size
+                docs=documents, verbose=False,
+                query_str_params=query_str_params, batch_size=client_batch_size, base_body = base_body
             )
 
         else:
@@ -400,53 +393,22 @@ class Index:
             # ADD DOCS TIMER-LOGGER (2)
             start_time_client_request = timer()
 
-            if update_method == 'update':
-                res = self.http.put(path=path_with_query_str, body=documents)
-            elif update_method == 'replace':
-                res = self.http.post(path=path_with_query_str, body=documents)
-            else:
-                raise ValueError(f'Received unknown update_method `{update_method}`. '
-                                 f'Allowed update_methods: ["replace", "update"] ')
-
-
+            body = {"documents": documents, **base_body}
+            res = self.http.post(
+                path=path_with_query_str, body=body, index_name=self.index_name,
+            )
             end_time_client_request = timer()
             total_client_request_time = end_time_client_request - start_time_client_request
 
             mq_logger.debug(f"add_documents roundtrip: took {(total_client_request_time):.3f}s to send {num_docs} "
                             f"docs to Marqo (roundtrip, unbatched), for an average of {(total_client_request_time / num_docs):.3f}s per doc.")
             errors_detected = False
-            if server_batch_size is not None:
-                # with Server Batching (show processing time for each batch)
-                mq_logger.debug(f"add_documents Marqo index (server-side batch reports): ")
 
-                if processes is not None and processes > 1:
-                    # for multiprocess, timing messages should be arranged by process, then batch
-                    for process in range(len(res)):
-                        mq_logger.debug(f"   process {process}:")
-
-                        for batch in range(len(res[process])):
-                            server_batch_result_count = len(res[process][batch]["items"])
-                            mq_logger.debug(f"       marqo server batch {batch}: "
-                                            f"processed {server_batch_result_count} docs in {(res[process][batch]['processingTimeMs'] / 1000):.3f}s, "
-                                            f"for an average of {(res[process][batch]['processingTimeMs'] / (1000 * server_batch_result_count)):.3f}s per doc.")
-                            if 'errors' in res[process][batch] and res[process][batch]['errors']:
-                                errors_detected = True
-                else:
-                    # for single process, timing messages should be arranged by batch ONLY
-                    for batch in range(len(res)):
-                        server_batch_result_count = len(res[batch]["items"])
-                        mq_logger.debug(f"   marqo server batch {batch}: "
-                                        f"processed {server_batch_result_count} docs in {(res[batch]['processingTimeMs'] / 1000):.3f}s, "
-                                        f"for an average of {(res[batch]['processingTimeMs'] / (1000 * server_batch_result_count)):.3f}s per doc.")
-                        if 'errors' in res[batch] and res[batch]['errors']:
-                            errors_detected = True
-            else:
-                # no Server Batching
-                if 'processingTimeMs' in res:       # Only outputs log if response is non-empty
-                    mq_logger.debug(f"add_documents Marqo index: took {(res['processingTimeMs'] / 1000):.3f}s for Marqo to process & index {num_docs} "
-                                    f"docs (server unbatched), for an average of {(res['processingTimeMs'] / (1000 * num_docs)):.3f}s per doc.")
-                if 'errors' in res and res['errors']:
-                    mq_logger.info(error_detected_message)
+            if 'processingTimeMs' in res:       # Only outputs log if response is non-empty
+                mq_logger.debug(f"add_documents Marqo index: took {(res['processingTimeMs'] / 1000):.3f}s for Marqo to process & index {num_docs} "
+                                f"docs (server unbatched), for an average of {(res['processingTimeMs'] / (1000 * num_docs)):.3f}s per doc.")
+            if 'errors' in res and res['errors']:
+                mq_logger.info(error_detected_message)
 
             if errors_detected:
                 mq_logger.info(error_detected_message)
@@ -467,13 +429,11 @@ class Index:
         base_path = f"indexes/{self.index_name}/documents/delete-batch"
         path_with_refresh = base_path if auto_refresh is None else base_path + f"?refresh={str(auto_refresh).lower()}"
 
-        return self.http.post(
-            path=path_with_refresh, body=ids
-        )
+        return self.http.post(path=path_with_refresh, body=ids, index_name=self.index_name,)
 
     def get_stats(self) -> Dict[str, Any]:
         """Get stats about the index"""
-        return self.http.get(path=f"indexes/{self.index_name}/stats")
+        return self.http.get(path=f"indexes/{self.index_name}/stats", index_name=self.index_name,)
 
     @staticmethod
     def _maybe_datetime(the_date: Optional[Union[datetime, str]]) -> Optional[datetime]:
@@ -490,9 +450,8 @@ class Index:
 
     def _batch_request(
             self, docs: List[Dict],  base_path: str,
-            query_str_params: str,
-            update_method: str, verbose: bool = True,
-            auto_refresh: bool = True, batch_size: int = 50
+            query_str_params: str, base_body: dict, verbose: bool = True,
+            auto_refresh: bool = True, batch_size: int = 50,
     ) -> List[Dict[str, Any]]:
         """Batches a large chunk of documents to be sent as multiple
         add_documents invocations
@@ -528,13 +487,9 @@ class Index:
             errors_detected = False
 
             t0 = timer()
-            if update_method == 'replace':
-                res = self.http.post(path=path_with_query_str, body=docs)
-            elif update_method == 'update':
-                res = self.http.put(path=path_with_query_str, body=docs)
-            else:
-                raise ValueError(f'Received unknown update_method `{update_method}`. '
-                                 f'Allowed update_methods: ["replace", "update"] ')
+            body = {"documents": docs, **base_body}
+            res = self.http.post(path=path_with_query_str, body=body, index_name=self.index_name)
+
             total_batch_time = timer() - t0
             num_docs = len(docs)
 
@@ -590,4 +545,4 @@ class Index:
 
     def get_settings(self) -> dict:
         """Get all settings of the index"""
-        return self.http.get(path=f"indexes/{self.index_name}/settings")
+        return self.http.get(path=f"indexes/{self.index_name}/settings", index_name=self.index_name,)
