@@ -22,14 +22,11 @@ class SearchBody(BaseMarqoModel):
     image_download_headers: Optional[Dict] = None
     context: Optional[Dict] = None
     scoreModifiers: Optional[Dict] = None
-
+    modelAuth: Optional[Dict] = None
 
 class BulkSearchBody(SearchBody):
     index: str
-    # Attributes that are not supported in bulk search
-    context: None = None
-    scoreModifiers: None = None
 
 
 class BulkSearchQuery(BaseMarqoModel):
-    queries: List[BulkSearchBody]
\ No newline at end of file
+    queries: List[BulkSearchBody]
